unit novySF;

interface

uses
  Windows, Messages, SysUtils, Classes, Graphics, Controls, Forms, Dialogs,
  StdCtrls, ExtCtrls;

type
  TFormat = (NEXUS, FASTA, Phylip);
  TStates = array of record
    Symbol: char;
    Order: integer;
  end;
  TForm1 = class(TForm)
    Button1: TButton;
    OpenDialog1: TOpenDialog;
    OpenDialog2: TOpenDialog;
    Button2: TButton;
    Label1: TLabel;
    Image1: TImage;
    Button3: TButton;
    RadioGroup1: TRadioGroup;
    RadioGroup2: TRadioGroup;
    Label2: TLabel;
    Memo1: TMemo;
    Button4: TButton;
    OpenDialog3: TOpenDialog;
    Button5: TButton;
    procedure Button1Click(Sender: TObject);
    procedure Button2Click(Sender: TObject);
    procedure DrawTree;
    procedure DrawGroup (GroupNumber: integer);
    procedure Image1MouseDown(Sender: TObject; Button: TMouseButton;
      Shift: TShiftState; X, Y: Integer);
    procedure Button3Click(Sender: TObject);
    procedure Button4Click(Sender: TObject);
    procedure Button5Click(Sender: TObject);
  private
    { Private declarations }
  public
    { Public declarations }
  end;

var
  Form1: TForm1;
  AlignmentFile: TextFile;
  AlignmentFileName, LogFileName: String;
  Taxon: array of record
    Name: String;
    Data: array of Char;
  end;
  Group: array of record
    Level: integer;
    OpenToInclude: boolean;
    IncludedGroups: array of integer;
    Taxa: array of integer;
    X1Border, X2Border, Y1Border, Y2Border: integer;
    Chosen: boolean;
    PositionStates: array of char;
  end;
  PositionChanges: array of integer;
  MaxGroupLevel: integer = 0;
  MinChanges: integer;
  InputFormat: TFormat;
  DataType: (DNA, AA);
  SymbolsUsed, CountChangesSymbols: array of char;
  UsedGroups: array of string;
  CanResizeTree: boolean = false;
  AssignedTaxa: array of integer;
  ChangesLoaded: boolean = false;

implementation

{$R *.DFM}

//Checking for non-defined symbols in DNA or protein alignments
function SymbolControl (ControlledSymbol: char; TaxonName: string;
    PositionNumber: integer): boolean;
begin
  result:=true;
  case DataType of
    DNA: case ControlledSymbol of
      'A', 'T', 'C', 'G', 'a', 't', 'c', 'g', 'U', 'u', 'M', 'R', 'W', 'S',
          'Y', 'K', 'm', 'r', 'w', 's', 'y', 'k', 'V', 'H', 'D', 'B', 'v', 'h',
          'd', 'b', 'N', 'n', '-', '?':;
      else
      begin
        MessageDlg ('Symbol "' + ControlledSymbol +
            '" (taxon "' + TaxonName + '", position ' + IntToStr (PositionNumber)
            + ') is not defined for DNA alignments.',
            mtWarning, [mbOK], 0);
        result:=false;
      end;
    end;
    AA: case ControlledSymbol of
      'A', 'B', 'C', 'D', 'E', 'F', 'G', 'H', 'I', 'K', 'L', 'M', 'N', 'P',
      'Q', 'R', 'S', 'T', 'U', 'V', 'W', 'X', 'Y', 'Z', 'a', 'b', 'c',
      'd', 'e', 'f', 'g', 'h', 'i', 'k', 'l', 'm', 'n', 'p', 'q', 'r', 's',
      't', 'u', 'v', 'w', 'x', 'y', 'z', '-', '?':;
      else
      begin
        MessageDlg ('Symbol "' + ControlledSymbol +
            '" (taxon "' + TaxonName + '", position ' + IntToStr (PositionNumber)
            + ') is not defined for protein alignments.',
            mtWarning, [mbOK], 0);
        result:=false;
      end;
    end;
  end;
end;

//Loads nexus file
function LoadNexusFile: boolean;
var Symbol: char;
  TaxonCounter, PositionCounter, AlignmentLength: integer;
  Word: String;
  ReadWord: boolean;
begin
  result:=true;
  InputFormat:=NEXUS;
  ReadWord:=False;
  Reset (AlignmentFile);

//reading of "ntax", "nchar" values, waiting for "matrix"
  repeat
    Read (AlignmentFile, Symbol);
    if (symbol = 'm') or (symbol = 'n') or (symbol = 'M') or (symbol = 'N') then
    begin
      Word:='';
      ReadWord:=true;
    end;
    if length (Word) > 10 then ReadWord:=False;
    if ReadWord=True then Word:=Word + Symbol;
    if (Word = 'ntax=') or (Word='ntax = ') then
    begin
      Word:='';
      Read (AlignmentFile, Symbol);
      repeat
        Word:=Word+Symbol;
        if eoln (AlignmentFile) then break;
        Read (AlignmentFile, Symbol)
      until (Symbol = ' ') or (Symbol = ';');
      Setlength (Taxon, StrToInt (Word));
    end;
    if (Word = 'nchar=') or (Word='nchar = ') then
    begin
      Word:='';
      Read (AlignmentFile, Symbol);
      repeat
        Word:=Word+Symbol;
        if eoln (AlignmentFile) then break;
        Read (AlignmentFile, Symbol);
      until (Symbol = ' ') or (Symbol = ';');
      AlignmentLength:= StrToInt (Word);
    end;
  until (Word = 'matrix') or (Word = 'Matrix') or (Word = 'MATRIX');

//data reading for all taxa
  Readln (AlignmentFile);
  for TaxonCounter:= 1 to Length (Taxon) do
  begin
    Word:='';
    PositionCounter:=0;
    while eoln (AlignmentFile) do
    begin
      readln (AlignmentFile);
      if eof (AlignmentFile) then exit;
    end;
    repeat
      read (AlignmentFile, Symbol);
      if Symbol <> ' ' then Word:=Word+Symbol;
    until (Symbol = ' ') or eoln (AlignmentFile);
    Taxon[TaxonCounter-1].Name:=Word;
    Setlength (Taxon[TaxonCounter -1].Data, AlignmentLength);
    repeat
      while eoln (AlignmentFile) do
      begin
        readln (AlignmentFile);
        if eof (AlignmentFile) then exit;
      end;
      Read (AlignmentFile, Symbol);
      case Symbol of
        ' ':;
        else
        begin
          PositionCounter:=PositionCounter+1;
          if SymbolControl (Symbol, taxon[TaxonCounter-1].Name, Positioncounter) = false then
          begin
            result:=false;
            CloseFile (AlignmentFile);
            exit;
          end;
          Taxon[TaxonCounter-1].Data[PositionCounter-1]:=Symbol;
        end;
      end;
    until PositionCounter=AlignmentLength;
    Readln (AlignmentFile);
  end;
  CloseFile (AlignmentFile);
end;

//Loads fasta file
function LoadFastaFile: boolean;
var Symbol: Char;
  TaxonCounter, PositionCounter: integer;
begin
  result:=true;
  InputFormat:=FASTA;
  TaxonCounter:=0;
  Reset (AlignmentFile);
  repeat
    while eoln (AlignmentFile) do
    begin
      readln (AlignmentFile);
      if eof (AlignmentFile) then exit;
    end;
    Read (AlignmentFile, Symbol);
    if symbol = '>' then
    begin
      TaxonCounter:= TaxonCounter + 1;
      SetLength (Taxon, TaxonCounter);
      repeat
        Read (AlignmentFile, Symbol);
        if Symbol <> ' ' then Taxon[TaxonCounter-1].Name:=Taxon[TaxonCounter-1].Name+Symbol;
      until eoln (AlignmentFile);
      PositionCounter:=0;
    end
    else
    begin
      if SymbolControl (Symbol, Taxon[TaxonCounter-1].Name, PositionCounter+1) = false then
      begin
        result:=false;
        CloseFile (AlignmentFile);
        exit;
      end;
      PositionCounter:=PositionCounter+1;
      SetLength (Taxon[TaxonCounter-1].Data,PositionCounter);
      Taxon[TaxonCounter-1].Data[PositionCounter-1]:=Symbol;
    end;
  until eof (AlignmentFile);
  CloseFile (AlignmentFile);
end;

//Loads Phylip file
function LoadPhylipFile: boolean;
var Symbol: Char;
  TaxonCounter, DataCounter, LastData: integer;
  Word: string;
begin
  Word:='';
  result:=true;
  InputFormat:=Phylip;
  Reset (AlignmentFile);

//reading of the first part of file (with taxon names)
  begin
    while eoln (AlignmentFile) do
    begin
      readln (AlignmentFile);
      if eof (AlignmentFile) then exit;
    end;
    Read (AlignmentFile, Symbol);
    Read (AlignmentFile, Symbol);
    Word:=Symbol;
    Read (AlignmentFile, Symbol);
    while Symbol <> ' ' do
    begin
      Word:=Word + Symbol;
      read (AlignmentFile, Symbol);
    end;
    setlength (Taxon, strtoint (Word));
    Word:='';
    repeat
      Read (Alignmentfile, Symbol);
      Word:=Word + Symbol;
    until eoln (AlignmentFile);
    Readln (AlignmentFile);
    for TaxonCounter:=0 to high (Taxon) do setlength (Taxon[TaxonCounter].Data, strtoint (Word));
    for TaxonCounter:=0 to high (taxon) do
    begin
      Word:='';
      Read (Alignmentfile, Symbol);
      while Symbol <> ' ' do
      begin
        Word:=Word + Symbol;
        read (AlignmentFile, Symbol);
      end;
      Taxon[Taxoncounter].Name:=Word;
      Word:='';
      for DataCounter:=0 to 49 do
      begin
        if DataCounter = length (Taxon[0].Data) then break;
        repeat read (Alignmentfile, Symbol) until Symbol <> ' ';
        begin
          if SymbolControl (Symbol, Taxon[TaxonCounter].Name, DataCounter+1) = false then
          begin
            result:=false;
            CloseFile (AlignmentFile);
            exit;
          end;
          Taxon[TaxonCounter].Data[DataCounter]:=Symbol;
        end;
      end;
      Readln (alignmentFile);
    end;
  end;

//reading of the rest of file
  repeat
    Readln (AlignmentFile);
    LastData:=DataCounter;
    for TaxonCounter:=0 to high (Taxon) do
    begin
      for Datacounter:=LastData to LastData + 49 do
      begin
        if DataCounter = length (Taxon[0].Data) then break;
        repeat read (Alignmentfile, Symbol) until Symbol <> ' ';
        begin
          if SymbolControl (Symbol, Taxon[TaxonCounter].Name, DataCounter+1) = false then
          begin
            result:=false;
            CloseFile (AlignmentFile);
            exit;
          end;
          Taxon[TaxonCounter].Data[DataCounter]:=Symbol;
        end;
      end;
      Readln (AlignmentFile);
    end;
  until eof (AlignmentFile);
end;

//writes number of taxa and alignmnent lenght, names of all taxa
//and data for the first taxon in logfile
Procedure LoadAlignmentLogFile;
var LogFile: TextFile;
  counter: integer;
begin;
  LogFileName:=AlignmentFileName;
  for counter:= length(LogFileName)-1 downto 0 do
      if LogFileName[counter] = '.' then
  begin
    LogFileName:=copy(LogFileName,0,counter-1) + 'Log.txt';
    break;
  end
  else if counter=0 then LogFileName:=LogFileName+ 'Log.txt';
  Assignfile (LogFile, LogFileName);
  Rewrite (LogFile);
  Writeln (LogFile, 'Number of taxa: ' + IntToStr(Length (Taxon)));
  Writeln (LogFile, 'Alignment length: ' + IntToStr(Length (Taxon[0].Data)));
  Writeln (LogFile);
  Writeln (LogFile, 'Taxa:');
  for counter:=1 to length (Taxon) do
      if counter = length (Taxon) then Writeln (LogFile, Taxon[counter-1].Name)
      else Write (LogFile, Taxon[counter-1].Name + ', ');
  Writeln (LogFile);
  CloseFile (LogFile);
end;

//checks for duplicated taxon names and identical data length (relevant for
//FASTA only
function control: boolean;
var counter, counter2: integer;
begin
  result:=true;
  for counter:=0 to high (Taxon) - 1 do
  begin
    if InputFormat = FASTA then if length (taxon[counter+1].Data) <> length (Taxon[0].Data) then
    begin
      MessageDlg ('Data length must be the same for all taxa (taxon "'
          + Taxon[0].Name + '": ' + inttostr(length (Taxon[0].Data)) + ' positions, taxon "'
          + Taxon[counter+1].Name + '": ' + inttostr (length (Taxon[counter+1].Data))
          + ' positions).',mtError, [mbOK], 0);
      result:=false;
      exit;
    end;
    for counter2:=counter + 1 to high (Taxon) do
        if Taxon[counter].Name = Taxon[counter2].name then
    begin
      MessageDlg ('Taxon names must be unique (check taxa named "' +
          Taxon[counter].Name + '").', mtError, [mbOK], 0);
      result:=false;
    end;
  end;
end;

//manages the clicking on Load Alignment button
procedure TForm1.Button1Click(Sender: TObject);
var FirstSymbol: Char;
begin
  case RadioGroup1.ItemIndex of
  0: DataType:=DNA;
  1: Datatype:=AA;
  end;
  if OpenDialog1.Execute then
  begin
    AlignmentFileName:=OpenDialog1.FileName;
    AssignFile (AlignmentFile, AlignmentFileName);
    Reset (AlignmentFile);
    Read (AlignmentFile, FirstSymbol);
    CloseFile (AlignmentFile);
    if FirstSymbol = '#' then
    begin
      if LoadNexusFile = false then
      begin
        Taxon:=nil;
        exit;
      end;
    end
    else if FirstSymbol = '>' then
    begin
      if LoadFastaFile = false then
      begin
        taxon:=nil;
        exit;
      end;
    end
    else if FirstSymbol = ' ' then
    begin
      if LoadPhylipFile = false then
      begin
        taxon:=nil;
        exit;
      end;
    end
    else
    begin
      MessageDlg
          ('Alignment file must begin with "#" (NEXUS file) or ">" (FASTA file) of " " (Phylip file).',
          mtError, [mbOK], 0);
      exit;
    end;
    if Control = false then
    begin
      Taxon:=nil;
      exit;
    end;
    Button1.Enabled:=False;
    Button2.Enabled:=True;
    Button4.Enabled:=True;    
    RadioGroup1.Enabled:=false;
    Image1.Height:=10000;
    Image1.Canvas.Rectangle (0,0,Image1.Width, Image1.Height);
    Application.ProcessMessages;
    Image1.Visible:=false;
    LoadAlignmentLogFile;
    Memo1.Lines.Clear;
    Memo1.Lines.Append('  Hints:');
    Memo1.Lines.Append(' ');
    Memo1.Lines.Append('The second step is loading your');
    Memo1.Lines.Append('input tree. Use the Newick');
    Memo1.Lines.Append('(bracketed) format.');
    Memo1.Lines.Append(' ');
    Memo1.Lines.Append('Note that polytomies are allowed.');
    Memo1.Lines.Append(' ');
    Memo1.Lines.Append('(An alternative is loading numbers of');
    Memo1.Lines.Append('changes directly from a file. Use a');
    Memo1.Lines.Append('succession of numbers separated');
    Memo1.Lines.Append('by whitespace - first number');
    Memo1.Lines.Append('corresponds to a number of changes');
    Memo1.Lines.Append('(or, more generally, to a substitution');
    Memo1.Lines.Append('rate) of the first alignment position');
    Memo1.Lines.Append('etc. See sample file.)');
  end;
end;

//connects taxon names from tree file with taxon names obtained from alignment
//file, creates a group containig this taxon and includes this one-taxon group
//in a proper higher group
function AssignTaxon (TaxonName: string): boolean;
var TaxonNumber, TaxonCounter, GroupCounter: integer;
  found: boolean;
begin
  result:=true;
  found:=false;
  for Taxoncounter:=0 to high (Taxon) do
      if TaxonName = Taxon[TaxonCounter].Name then
  begin
    found:=true;
    if AssignedTaxa[TaxonCounter] = 1 then
    begin
      MessageDlg
          ('Taxon "' + TaxonName + '" is mentioned twice in your tree file.',
          mtError, [mbOK], 0);
      result:=false;
      exit;
    end
    else AssignedTaxa[TaxonCounter] := 1;
  end;
  if found = false then begin
    MessageDlg
        ('Taxon "' + TaxonName + '" is not contained in alignment file. Note: the program is case sensitive.',
        mtError, [mbOK], 0);
    result:=false;
    exit;
  end;
  for TaxonCounter:=0 to high (Taxon) do
      if TaxonName = Taxon[TaxonCounter].Name then
  begin
    TaxonNumber:=TaxonCounter;
    break;
  end;
  SetLength (Group, length(Group)+1);
  SetLength (Group[high(Group)].Taxa, 1);
  Group[high(Group)].Taxa[0]:=TaxonNumber;
  for GroupCounter:= high (Group)-1 downto 0 do
      if Group[GroupCounter].OpenToInclude = true then
  begin
    SetLength (Group[GroupCounter].IncludedGroups,
        Length(Group[GroupCounter].IncludedGroups)+1);
    Group[GroupCounter].IncludedGroups[high(Group[GroupCounter].IncludedGroups)]
        :=high(Group);
    Group[high(Group)].Level:=Group[GroupCounter].Level+1;
    Break;
  end;
end;

//Draws a branch of group[GroupNumber], then does the same for all subgroups of
//the group
Procedure TForm1.DrawGroup (GroupNumber: integer);
var counter: integer;
begin
  if group[GroupNumber].chosen=true then Image1.Canvas.Pen.Color:=clRed
      else Image1.Canvas.Pen.Color:=clBlack;
  Image1.Canvas.MoveTo(Group[GroupNumber].X2Border,Group[GroupNumber].Y1Border);
  Image1.Canvas.LineTo(Group[GroupNumber].X2Border,Group[GroupNumber].Y2Border);
  Image1.Canvas.MoveTo(Group[GroupNumber].X1Border,
      (Group[GroupNumber].Y1Border+Group[GroupNumber].Y2Border) div 2);
  Image1.Canvas.LineTo(Group[GroupNumber].X2Border,
      (Group[GroupNumber].Y1Border+Group[GroupNumber].Y2Border) div 2);
  if length (Group[GroupNumber].IncludedGroups) > 0 then
      for counter := 0 to high (Group[GroupNumber].IncludedGroups) do
      Drawgroup (Group[GroupNumber].IncludedGroups[counter]);
end;


//writes taxon names in the image, counts where in the image will be the
//borders of branches of all groups, then calls procedure DrawGroup for
//the first, all-other-groups-containing group
Procedure TForm1.DrawTree;
var TextYPos, GroupCounter: integer;
begin
  Image1.Height:=25*(length(Taxon)+1);
  Image1.Canvas.Pen.Width:=4;
  Image1.Canvas.Font.Size:=12;
  TextYPos:=Image1.Height-35;
  for GroupCounter:= high (Group) downto 0 do
  begin
    if length (Group[GroupCounter].Taxa)=1 then
    begin
      Image1.Canvas.TextOut (Image1.width-100, TextYPos,
          Taxon[Group[GroupCounter].Taxa[0]].Name);
      Group[GroupCounter].X2Border:=Image1.Width-110;
      Group[GroupCounter].Y1Border:=Image1.Canvas.PenPos.Y+10;
      Group[GroupCounter].Y2Border:=Image1.Canvas.PenPos.Y+10;
      Group[GroupCounter].X1Border:=20+(Image1.Width-130) div MaxGroupLevel *
          Group[GroupCounter].Level;
      TextYPos:=TextYPos-25;
    end
    else
    begin
      Group[GroupCounter].X2Border:=Group[Group[GroupCounter].
          IncludedGroups[0]].X1Border;
      Group[GroupCounter].Y1Border:= (Group[Group[GroupCounter].
          IncludedGroups[0]].Y1Border+Group[Group[GroupCounter].
          IncludedGroups[0]].Y2Border) div 2;
      Group[GroupCounter].Y2Border:= (Group[Group[GroupCounter].
          IncludedGroups[high(Group[GroupCounter].
          IncludedGroups)]].Y1Border+Group[Group[GroupCounter].
          IncludedGroups[high(Group[GroupCounter].
          IncludedGroups)]].Y2Border) div 2;
      Group[GroupCounter].X1Border:=20+(Image1.Width-130) div MaxGroupLevel *
          Group[GroupCounter].Level;
    end;
  end;
  DrawGroup (0);
end;

//manages the clicking on Load Tree button. Creates all groups and orders them
//hierarchicaly so that subgroups are included in proper groups. also reads the
//names of taxa and calls procedure AssignTaxon for them.
procedure TForm1.Button2Click(Sender: TObject);
var TreeFile: TextFile;
  Symbol: char;
  reading, assigned: boolean;
  counter, GroupCounter, GroupLevel: integer;
  Word: String;
begin
  CanResizeTree:=True;
  if OpenDialog2.Execute then
  begin
    setlength (AssignedTaxa, length (Taxon));
    GroupLevel:=-1;
    AssignFile (TreeFile, OpenDialog2.FileName);
    Reset (TreeFile);
    Read (TreeFile, Symbol);
    If Symbol <> '(' then begin
      MessageDlg
          ('Tree file must begin with "(" (followed with otgroup).',
          mtError, [mbOK], 0);
      CloseFile (TreeFile);
      exit;
    end;
    Reset (TreeFile);
    repeat
      if not eof (TreeFile) then if eoln (TreeFile) then
      begin
        readln (TreeFile);
        continue;
      end;
      read (TreeFile, Symbol);
      case symbol of
        ';':;
        '(':
        begin
          for GroupCounter:= high (Group) downto 0 do
              if Group[GroupCounter].OpenToInclude=true then
          begin
            SetLength (Group[GroupCounter].IncludedGroups,
                Length(Group[GroupCounter].IncludedGroups)+1);
            Group[GroupCounter].IncludedGroups[high(Group[GroupCounter].
                IncludedGroups)]:=high(Group)+1;
            break;
          end;
          GroupLevel:=GroupLevel+1;
          if MaxGroupLevel < Grouplevel + 2 then MaxGroupLevel:=GroupLevel+2;
          Setlength(Group, Length (Group)+1);
          Group[high(Group)].OpenToInclude:=true;
          Group[high(Group)].Level:=GroupLevel;
          reading:=true;
        end;
        ':':
        begin
          reading:=false;
          if Assigned=false then if AssignTaxon (Word) = false then begin
            CloseFile (TreeFile);
            group:=nil;
            AssignedTaxa:=nil;
            exit;
          end;
          Assigned:=true;
          Word:='';
        end;
        ',':
        begin
          if (reading=true) and (assigned=false) then if AssignTaxon (Word) = false then begin
            CloseFile (TreeFile);
            group:=nil;
            AssignedTaxa:=nil;
            exit;
          end;;
          assigned:=true;
          reading:=true;
          Word:='';
        end;
        ')':
        begin
          GroupLevel:=GroupLevel-1;
          if reading=true and (assigned=false) then if AssignTaxon (Word) = false then begin
            CloseFile (TreeFile);
            group:=nil;
            AssignedTaxa:=nil;            
            exit;
          end;
          assigned:=true;
          Word:='';
          reading:=true;
          for GroupCounter:=high(Group) downto 0 do
              if Group[GroupCounter].OpenToInclude=true then
          begin
            Group[GroupCounter].OpenToInclude:=false;
            break;
          end;
        end;
        else
        begin
          if reading=true then if Ord(Symbol) <> 39 then Word:=Word+Symbol;
          assigned:=false;
        end;
      end;
    until eof (TreeFile);
    CloseFile (TreeFile);
    for counter:=0 to high (AssignedTaxa) do
        if AssignedTaxa[counter]=0 then
    begin
      MessageDlg
          ('Taxon "' + Taxon[counter].Name + '" is not contained in your tree file. Note: the program is case sensitive.',
          mtError, [mbOK], 0);
      Group:=nil;
      AssignedTaxa:=nil;
      exit;
    end;
    AssignedTaxa:=nil;
    Button2.Enabled:=False;
    Button4.Enabled:=False;    
    Label1.Enabled:=true;
    Image1.Visible:=true;
    Memo1.Lines.Clear;
    Memo1.Lines.Append('  Hints:');
    Memo1.Lines.Append(' ');
    Memo1.Lines.Append('When the tree is depicted in the');
    Memo1.Lines.Append('image window, click directly on');
    Memo1.Lines.Append('branches you want to choose');
    Memo1.Lines.Append('as monophyla used for changes');
    Memo1.Lines.Append('counting. Chosen groups will');
    Memo1.Lines.Append('become red.');
    Memo1.Lines.Append(' ');
    Memo1.Lines.Append('Once you have finished your');
    Memo1.Lines.Append('choice, specify the format you');
    Memo1.Lines.Append('want your new datasets saved');
    Memo1.Lines.Append('in and click on the New alignments');
    Memo1.Lines.Append('button.');
    Memo1.Lines.Append(' ');
    Memo1.Lines.Append('[You can both choose and');
    Memo1.Lines.Append('dechoose branches, creating');
    Memo1.Lines.Append('e.g. paraphyletic groups to');
    Memo1.Lines.Append('be used, you can also');
    Memo1.Lines.Append('dechoose some species.]');
    Memo1.Lines.Append(' ');
    Memo1.Lines.Append('[Note that choosing/dechoosing');
    Memo1.Lines.Append('a branch will choose/decoose all');
    Memo1.Lines.Append('its subbranches.]');
    Memo1.Lines.Append(' ');
    Memo1.Lines.Append('[Do not worry to try it several');
    Memo1.Lines.Append('times. Clicking twice on the');
    Memo1.Lines.Append('basalmost branch (whole tree)');
    Memo1.Lines.Append('will reset the tree.]');
    DrawTree;
  end;
end;

//Sets the "chosen" property (true for groups chosen for counting of changes
//within them) of group[GroupNumber] to true or false. Then does the same for
//all groups it includes.
procedure ChooseGroup (GroupNumber: integer; YesOrNo: boolean);
var counter: integer;
begin
  Group[GroupNumber].Chosen := YesOrNo;
  if length (Group[GroupNumber].IncludedGroups) > 0 then
  begin
    for counter := 0 to high (Group[GroupNumber].IncludedGroups) do
        ChooseGroup (Group[GroupNumber].IncludedGroups[counter], YesOrNo);
  end;
end;

//manages clicking in the image. Locates a group branch of which was clicked,
//than calls ChooseGroup procedure for it.
procedure TForm1.Image1MouseDown(Sender: TObject; Button: TMouseButton;
  Shift: TShiftState; X, Y: Integer);
var counter: integer;
begin
  for counter:=0 to high (group) do
      if (X>group[counter].X1Border) and (X<group[counter].X2Border)
      and (Y>group[counter].Y1Border-4) and (Y<group[counter].Y2Border+4) then
  begin
    if group[counter].chosen=true then ChooseGroup (counter, false)
        else ChooseGroup (counter, true);
    DrawGroup (counter);
    Button3.Enabled:=true;
    RadioGroup2.Enabled:=true;
    break;
  end;
end;

//Sets initial states for all top-most, taxa-containing groups according to
//standard code (DNA).
procedure SetStateDNA (GroupNumber: integer; Position: integer);
begin
  case Taxon[group[GroupNumber].Taxa[0]].Data[Position] of
    'A', 'a':
    begin
      Setlength (Group[GroupNumber].PositionStates, 1);
      Group[GroupNumber].PositionStates[0]:='A';
    end;
    'U', 'u', 'T', 't':
    begin
      Setlength (Group[GroupNumber].PositionStates, 1);
      Group[GroupNumber].PositionStates[0]:='T';
    end;
    'C', 'c':
    begin
      Setlength (Group[GroupNumber].PositionStates, 1);
      Group[GroupNumber].PositionStates[0]:='C';
    end;
    'G', 'g':
    begin
      Setlength (Group[GroupNumber].PositionStates, 1);
      Group[GroupNumber].PositionStates[0]:='G';
    end;
    'M', 'm':
    begin
      Setlength (Group[GroupNumber].PositionStates, 2);
      Group[GroupNumber].PositionStates[0]:='A';
      Group[GroupNumber].PositionStates[1]:='C';
    end;
    'R', 'r':
    begin
      Setlength (Group[GroupNumber].PositionStates, 2);
      Group[GroupNumber].PositionStates[0]:='A';
      Group[GroupNumber].PositionStates[1]:='G';
    end;
    'W', 'w':
    begin
      Setlength (Group[GroupNumber].PositionStates, 2);
      Group[GroupNumber].PositionStates[0]:='A';
      Group[GroupNumber].PositionStates[1]:='T';
    end;
    'S', 's':
    begin
      Setlength (Group[GroupNumber].PositionStates, 2);
      Group[GroupNumber].PositionStates[0]:='G';
      Group[GroupNumber].PositionStates[1]:='C';
    end;
    'K', 'k':
    begin
      Setlength (Group[GroupNumber].PositionStates, 2);
      Group[GroupNumber].PositionStates[0]:='G';
      Group[GroupNumber].PositionStates[1]:='T';
    end;
    'Y', 'y':
    begin
      Setlength (Group[GroupNumber].PositionStates, 2);
      Group[GroupNumber].PositionStates[0]:='T';
      Group[GroupNumber].PositionStates[1]:='C';
    end;
    'V', 'v':
    begin
      Setlength (Group[GroupNumber].PositionStates, 3);
      Group[GroupNumber].PositionStates[0]:='A';
      Group[GroupNumber].PositionStates[1]:='C';
      Group[GroupNumber].PositionStates[2]:='G';
    end;
    'H', 'h':
    begin
      Setlength (Group[GroupNumber].PositionStates, 3);
      Group[GroupNumber].PositionStates[0]:='A';
      Group[GroupNumber].PositionStates[1]:='C';
      Group[GroupNumber].PositionStates[2]:='T';
    end;
    'D', 'd':
    begin
      Setlength (Group[GroupNumber].PositionStates, 3);
      Group[GroupNumber].PositionStates[0]:='A';
      Group[GroupNumber].PositionStates[1]:='T';
      Group[GroupNumber].PositionStates[1]:='G';
    end;
    'B', 'b':
    begin
      Setlength (Group[GroupNumber].PositionStates, 3);
      Group[GroupNumber].PositionStates[0]:='T';
      Group[GroupNumber].PositionStates[1]:='C';
      Group[GroupNumber].PositionStates[2]:='G';
    end;
    'N', 'n':
    begin
      Setlength (Group[GroupNumber].PositionStates, 4);
      Group[GroupNumber].PositionStates[0]:='A';
      Group[GroupNumber].PositionStates[1]:='C';
      Group[GroupNumber].PositionStates[2]:='G';
      Group[GroupNumber].PositionStates[3]:='T';
    end;
    '?':
    begin
      Setlength (Group[GroupNumber].PositionStates, 5);
      Group[GroupNumber].PositionStates[0]:='A';
      Group[GroupNumber].PositionStates[1]:='C';
      Group[GroupNumber].PositionStates[2]:='G';
      Group[GroupNumber].PositionStates[3]:='T';
      Group[GroupNumber].PositionStates[4]:='-';
    end;
    '-':
    begin
      Setlength (Group[GroupNumber].PositionStates, 1);
      Group[GroupNumber].PositionStates[0]:='-';
    end;
  end;
end;

//Sets initial states for all top-most, taxa-containing groups according to
//standard code (proteins).
procedure SetStateAA (GroupNumber: integer; Position: integer);
begin
  case Taxon[group[GroupNumber].Taxa[0]].Data[Position] of
    'A', 'a':
    begin
      Setlength (Group[GroupNumber].PositionStates, 1);
      Group[GroupNumber].PositionStates[0]:='A';
    end;
    'B', 'b':
    begin
      Setlength (Group[GroupNumber].PositionStates, 2);
      Group[GroupNumber].PositionStates[0]:='D';
      Group[GroupNumber].PositionStates[1]:='N';
    end;
    'C', 'c':
    begin
      Setlength (Group[GroupNumber].PositionStates, 1);
      Group[GroupNumber].PositionStates[0]:='C';
    end;
    'D', 'd':
    begin
      Setlength (Group[GroupNumber].PositionStates, 1);
      Group[GroupNumber].PositionStates[0]:='D';
    end;
    'E', 'e':
    begin
      Setlength (Group[GroupNumber].PositionStates, 1);
      Group[GroupNumber].PositionStates[0]:='E';
    end;
    'F', 'f':
    begin
      Setlength (Group[GroupNumber].PositionStates, 1);
      Group[GroupNumber].PositionStates[0]:='F';
    end;
    'G', 'g':
    begin
      Setlength (Group[GroupNumber].PositionStates, 1);
      Group[GroupNumber].PositionStates[0]:='G';
    end;
    'H', 'h':
    begin
      Setlength (Group[GroupNumber].PositionStates, 1);
      Group[GroupNumber].PositionStates[0]:='H';
    end;
    'I', 'i':
    begin
      Setlength (Group[GroupNumber].PositionStates, 1);
      Group[GroupNumber].PositionStates[0]:='I';
    end;
    'K', 'k':
    begin
      Setlength (Group[GroupNumber].PositionStates, 1);
      Group[GroupNumber].PositionStates[0]:='K';
    end;
    'L', 'l':
    begin
      Setlength (Group[GroupNumber].PositionStates, 1);
      Group[GroupNumber].PositionStates[0]:='L';
    end;
    'M', 'm':
    begin
      Setlength (Group[GroupNumber].PositionStates, 1);
      Group[GroupNumber].PositionStates[0]:='M';
    end;
    'N', 'n':
    begin
      Setlength (Group[GroupNumber].PositionStates, 1);
      Group[GroupNumber].PositionStates[0]:='N';
    end;
    'P', 'p':
    begin
      Setlength (Group[GroupNumber].PositionStates, 1);
      Group[GroupNumber].PositionStates[0]:='P';
    end;
    'Q', 'q':
    begin
      Setlength (Group[GroupNumber].PositionStates, 1);
      Group[GroupNumber].PositionStates[0]:='Q';
    end;
    'R', 'r':
    begin
      Setlength (Group[GroupNumber].PositionStates, 1);
      Group[GroupNumber].PositionStates[0]:='R';
    end;
    'S', 's':
    begin
      Setlength (Group[GroupNumber].PositionStates, 1);
      Group[GroupNumber].PositionStates[0]:='S';
    end;
    'T', 't':
    begin
      Setlength (Group[GroupNumber].PositionStates, 1);
      Group[GroupNumber].PositionStates[0]:='T';
    end;
    'U', 'u':
    begin
      Setlength (Group[GroupNumber].PositionStates, 1);
      Group[GroupNumber].PositionStates[0]:='U';
    end;
    'V', 'v':
    begin
      Setlength (Group[GroupNumber].PositionStates, 1);
      Group[GroupNumber].PositionStates[0]:='V';
    end;
    'W', 'w':
    begin
      Setlength (Group[GroupNumber].PositionStates, 1);
      Group[GroupNumber].PositionStates[0]:='W';
    end;
    'Y', 'y':
    begin
      Setlength (Group[GroupNumber].PositionStates, 1);
      Group[GroupNumber].PositionStates[0]:='Y';
    end;
    'Z', 'z':
    begin
      Setlength (Group[GroupNumber].PositionStates, 2);
      Group[GroupNumber].PositionStates[0]:='E';
      Group[GroupNumber].PositionStates[1]:='Q';
    end;
    'X', 'x':
    begin
      Setlength (Group[GroupNumber].PositionStates, 21);
      Group[GroupNumber].PositionStates[0]:='A';
      Group[GroupNumber].PositionStates[1]:='C';
      Group[GroupNumber].PositionStates[2]:='D';
      Group[GroupNumber].PositionStates[3]:='E';
      Group[GroupNumber].PositionStates[4]:='F';
      Group[GroupNumber].PositionStates[5]:='G';
      Group[GroupNumber].PositionStates[6]:='H';
      Group[GroupNumber].PositionStates[7]:='I';
      Group[GroupNumber].PositionStates[8]:='K';
      Group[GroupNumber].PositionStates[9]:='L';
      Group[GroupNumber].PositionStates[10]:='M';
      Group[GroupNumber].PositionStates[11]:='N';
      Group[GroupNumber].PositionStates[12]:='P';
      Group[GroupNumber].PositionStates[13]:='Q';
      Group[GroupNumber].PositionStates[14]:='R';
      Group[GroupNumber].PositionStates[15]:='S';
      Group[GroupNumber].PositionStates[16]:='T';
      Group[GroupNumber].PositionStates[17]:='U';
      Group[GroupNumber].PositionStates[18]:='V';
      Group[GroupNumber].PositionStates[19]:='W';
      Group[GroupNumber].PositionStates[20]:='Y';
    end;
    '?':
    begin
      Setlength (Group[GroupNumber].PositionStates, 22);
      Group[GroupNumber].PositionStates[0]:='A';
      Group[GroupNumber].PositionStates[1]:='C';
      Group[GroupNumber].PositionStates[2]:='D';
      Group[GroupNumber].PositionStates[3]:='E';
      Group[GroupNumber].PositionStates[4]:='F';
      Group[GroupNumber].PositionStates[5]:='G';
      Group[GroupNumber].PositionStates[6]:='H';
      Group[GroupNumber].PositionStates[7]:='I';
      Group[GroupNumber].PositionStates[8]:='K';
      Group[GroupNumber].PositionStates[9]:='L';
      Group[GroupNumber].PositionStates[10]:='M';
      Group[GroupNumber].PositionStates[11]:='N';
      Group[GroupNumber].PositionStates[12]:='P';
      Group[GroupNumber].PositionStates[13]:='Q';
      Group[GroupNumber].PositionStates[14]:='R';
      Group[GroupNumber].PositionStates[15]:='S';
      Group[GroupNumber].PositionStates[16]:='T';
      Group[GroupNumber].PositionStates[17]:='U';
      Group[GroupNumber].PositionStates[18]:='V';
      Group[GroupNumber].PositionStates[19]:='W';
      Group[GroupNumber].PositionStates[20]:='Y';
      Group[GroupNumber].PositionStates[21]:='-';
    end;
    '-':
    begin
      Setlength (Group[GroupNumber].PositionStates, 1);
      Group[GroupNumber].PositionStates[0]:='-';
    end;
  end;
end;

//if a group contains a single subgroup, its states are just copied
procedure CopyStates (GroupNumber: integer);
var counter: integer;
begin
  for counter:= 0 to high (Group[GroupNumber].IncludedGroups) do
      if Group[Group[GroupNumber].IncludedGroups[counter]].Chosen = true then
  begin
    Group[GroupNumber].PositionStates:=Group[Group[GroupNumber].
        IncludedGroups[counter]].PositionStates;
    exit;
  end;
end;

//checks for changes if just two subgroups are included in a group
procedure CheckChanges (GroupNumber: integer; Position: integer);
var counter, counter2, counter3: integer;
  checking: boolean;
  common: array of char;
begin
  checking:=false;

// first, it just loads the states of the first subgroup
  for counter:=0 to high (Group[GroupNumber].IncludedGroups) do
      if Group[Group[GroupNumber].Includedgroups[counter]].Chosen = true then
      if checking=false then
  begin
    checking:=true;
    Group[GroupNumber].PositionStates:=Group[Group[GroupNumber].
        IncludedGroups[counter]].PositionStates;
  end

  else

//then it checks for intersection of states of the two subgroups. If it is
//empty, a change is counted, state of the group set. If not empty, state of
//the group is set to the intersection.
  begin
    for counter2:=0 to high(Group[GroupNumber].PositionStates) do
        for counter3:= 0 to high(Group[Group[GroupNumber].
        IncludedGroups[counter]].PositionStates) do
        if Group[GroupNumber].PositionStates[counter2]=Group[Group[GroupNumber].
        IncludedGroups[counter]].PositionStates[counter3] then
    begin
      setlength (common, length (Common)+1);
      common[high(common)]:=Group[GroupNumber].PositionStates[counter2];
    end;
    if length (common) = 0 then
    begin
      setlength (Group[GroupNumber].PositionStates, length(Group[GroupNumber].
          PositionStates)+length(Group[Group[GroupNumber].
          IncludedGroups[counter]].PositionStates));
      for counter2:=length(Group[GroupNumber].PositionStates)-length(Group[Group
          [GroupNumber].Includedgroups[counter]].PositionStates) to high
          (Group[GroupNumber].PositionStates) do Group[GroupNumber].PositionStates
          [counter2]:=
          Group[Group[GroupNumber].Includedgroups[counter]].
          PositionStates[counter2-length(Group[GroupNumber].PositionStates)+
          length(Group[Group[GroupNumber].Includedgroups[counter]].
          PositionStates)];
      PositionChanges[Position]:=PositionChanges[Position]+1;
    end
    else
    begin
      setlength (group[groupNumber].PositionStates, length (common));
      for counter2:=0 to high (common) do
          Group[GroupNumber].Positionstates[counter2]:=common[counter2];
    end;
    exit;
  end;
end;

//Counts changes for some polytomies in the most general way.
function CountChanges (GroupNumber: integer; States: TStates): integer;
var DechosenGroups: array of integer;
  CountChangesChanges, counter, counter2, counter3, CurrentOrder: integer;
  checking, ChangeChecking: boolean;
begin
  CountChangesChanges:=-1;
  CountChangesSymbols:=copy(CountChangesSymbols, 0, 0);
  DechosenGroups:=copy(Dechosengroups, 0, 0);

//Goes through all relevant states in order set by procedure SetOrder.
//Eliminates all groups containing given state, if any is eliminated, a change
//is counted and the given symbol saved in CountChangesSymbols.
//When all groups are eliminated, the number of changes is generated.
  for CurrentOrder:= 0 to high (States) do
  begin
    for counter:=0 to high (States) do if
        States[counter].Order = CurrentOrder then
    begin
      ChangeChecking:=false;
      for counter2:= 0 to high (group[GroupNumber].IncludedGroups) do
          if group[group[GroupNumber].IncludedGroups[counter2]].chosen=true then
      begin
        checking:=false;
        for counter3:=0 to high (group[group[GroupNumber].
            IncludedGroups[counter2]].PositionStates) do if
            group[group[GroupNumber].IncludedGroups[counter2]].
            PositionStates[counter3] = States[counter].Symbol then
            checking:=true;
        if checking=true then
        begin
          group[group[GroupNumber].IncludedGroups[counter2]].chosen:=false;
          setlength(DechosenGroups, length (DechosenGroups)+1);
          Dechosengroups[high(DechosenGroups)]:=counter2;
          ChangeChecking:=true;
        end;
      end;
      if ChangeChecking=true then
      begin
        CountChangesChanges:=CountChangesChanges+1;
        setlength (CountChangesSymbols, length (CountChangesSymbols)+1);
        CountChangesSymbols[high(CountChangesSymbols)]:=States[counter].
            Symbol;
      end;
    end;
  end;
  CountChanges:=CountChangesChanges;
  for counter:=0 to high (Dechosengroups) do group[group[GroupNumber].
      IncludedGroups[counter]].Chosen:=true;
end;

//Goes through every possible order of symbols so that subgroups containing
//them are checked for them and minimum number of changes is obtained
procedure SetOrder (done, GroupNumber: integer; States: TStates);
var counter, counter2, counter3: integer;
  Actual: integer;
  checking: boolean;
  Used: array of integer;
begin

//if order for all states is already set, the CountChanges procedure is called
//to obtain the number of changes. If it is the lowest number yet, it is
//recorded as MinChanges and symbols from CountChangesSymbols copied to
//group states. If equal to Minchanges, its symbols are added to group states...
  if done > high (States) then
  begin
    CountChangesSymbols:=copy(CountChangesSymbols, 0, 0);
    Actual:=CountChanges (GroupNumber, States);
    if MinChanges = -1 then MinChanges:=Actual;
    if Actual<MinChanges then
    begin
      MinChanges:=Actual;
      setlength (group[GroupNumber].PositionStates,
          length (CountChangesSymbols));
      for counter:= 0 to high (CountChangesSymbols) do
          group[GroupNumber].PositionStates[counter]:=
          CountChangesSymbols[counter];
    end;
    if Actual=MinChanges then
    begin
      for counter:= 0 to high (CountChangesSymbols) do
      begin
        checking:=false;
        for counter2:= 0 to high (group[GroupNumber].PositionStates) do if
            CountChangesSymbols[counter] =
            group[GroupNumber].PositionStates[counter2]
            then checking:=true;
        if checking=false then
        begin
          setlength (group[GroupNumber].PositionStates, length (group
              [GroupNumber].PositionStates)+1);
          group[GroupNumber].PositionStates[high(group[GroupNumber].
              PositionStates)]:=CountChangesSymbols[counter];
        end;
      end;
    end;
  end

  else

//... else the procedure SetOrder calls itself so that every symbol is given
//every possible order at last.
  begin
    for counter:=0 to done-1 do
    begin
      setlength (Used, length (Used) +1);
      Used[high(Used)]:=States[counter].Order;
    end;
      for counter2:=0 to high (states) do
      begin
        checking:=false;
        for counter3:=0 to high (Used) do if Used[counter3]=counter2 then
            checking:=true;
        if checking = false then
        begin
          States[done].Order:=Counter2;
          done:=done+1;
          SetOrder (done, GroupNumber, States);
          done:=done-1;
        end;
      end;
  end;
end;

//tries to use several "shortcuts' to asses the number of changes within
//polytomies. Calls SetOrder when unable to continue.
procedure CheckChangesPoly (GroupNumber: integer; Position: integer);
var counter, counter2, counter3, counter4, counter5: integer;
  intersectionAdd, PassedChecking, checking, passed: boolean;
  common, commonLast, intersection, NewStates: array of char;
  States: TStates;
  TempDechosen: array of integer;
  LogFile: textfile;
begin
  TempDechosen:=copy(TempDechosen,0,0);
  common:=copy(common,0,0);
  commonLast:=copy(commonLast,0,0);
  checking:=false;
  for counter:=0 to high (Group[GroupNumber].IncludedGroups) do
      if Group[Group[GroupNumber].Includedgroups[counter]].Chosen = true then

//first, states of the first subgroup are loaded in "common"
      if checking=false then
  begin
    checking:=true;
    setlength (common, length
        (Group[Group[GroupNumber].IncludedGroups[counter]].PositionStates));
    for counter2:=0 to high (common) do
        common[counter2]:=
        Group[Group[GroupNumber].IncludedGroups[counter]].
        PositionStates[counter2];
  end
  else

//Then all the subgroups are checked whether intersection of their states is
//empty. If not, these common symbols are saved as states of group[GroupNumber].
  begin
    setlength (commonLast, length (common));
    for counter2:= 0 to high (common) do commonLast[counter2]:=common[counter2];
    common:=copy(common,0,0);
    for counter2:=0 to high (commonLast) do
        for counter3:= 0 to high(Group[Group[GroupNumber].
        IncludedGroups[counter]].PositionStates) do
        if commonLast[counter2]=Group[Group[GroupNumber].
        IncludedGroups[counter]].PositionStates[counter3] then
    begin
      setlength (common, length (common)+1);
      common[high(common)]:=commonLast[counter2];
    end;
    if length (common)=0 then break;
  end;
  if length (common) > 0 then
  begin
    setlength (group[groupNumber].PositionStates, length (common));
    for counter:=0 to high (common) do
        Group[GroupNumber].Positionstates[counter]:=common[counter];
    for counter:=0 to high (TempDechosen) do
        group[group[GroupNumber].IncludedGroups[TempDechosen[counter]]].
        Chosen:=true;
    exit;
  end

//if no symbols are common for all the subgroups, several "shortcuts" are
//applied
  else
  repeat
    checking:=false;

//the program now checks which symbols are common for any two subgroups...
    intersection:=copy(intersection, 0, 0);
    for counter:=0 to high (group[GroupNumber].IncludedGroups) do
        if group[group[GroupNumber].IncludedGroups[counter]].Chosen=true then
    begin
      for counter2:=counter to high (group[GroupNumber].IncludedGroups) do
          if counter <> counter2 then
          if group[group[GroupNumber].IncludedGroups[counter2]].Chosen=true then
          for counter3:=0 to high
          (group[group[GroupNumber].IncludedGroups[counter]].
          PositionStates) do
          for counter4:=0 to high
          (group[group[GroupNumber].IncludedGroups[counter2]].
          PositionStates) do
          if group[group[GroupNumber].IncludedGroups[counter]].
          PositionStates[counter3] =
          group[group[GroupNumber].IncludedGroups[counter2]].
          PositionStates[counter4] then
      begin
        checking:=true;
        intersectionAdd:=true;
        for counter5:=0 to high (intersection) do
            if intersection[counter5] = group[group[GroupNumber].
            IncludedGroups[counter]].PositionStates[counter3] then
            intersectionAdd:=false;
        if intersectionAdd = true then
        begin
          setlength (intersection, length(intersection)+1);
          intersection[high(intersection)] :=
              group[group[GroupNumber].IncludedGroups[counter]].
              PositionStates[counter3];
        end;
      end;
    end;

//... if no symbol is common for any two subgroups, then the number of changes
//equals to the number of subgroups - 1. States of every subgroup are copied to
//group[GroupNumber].Positionstates
    if checking=false then
    begin
      PositionChanges[Position]:=PositionChanges[Position]-1;
      for counter:=0 to high (group[GroupNumber].IncludedGroups) do
          if group[group[GroupNumber].IncludedGroups[counter]].Chosen=true then
      begin
        PositionChanges[Position]:=PositionChanges[Position]+1;
        for counter2:=0 to high(group[group[GroupNumber].
            IncludedGroups[counter]].PositionStates) do
        begin
          setlength (group[GroupNumber].PositionStates, length
              (group[GroupNumber].PositionStates)+1);
          group[GroupNumber].PositionStates[high(group[GroupNumber].
              PositionStates)]:=group[group[GroupNumber].
              IncludedGroups[counter]].PositionStates[counter2];
        end;
      end;
      for counter:=0 to high (TempDechosen) do
          group[group[GroupNumber].IncludedGroups[TempDechosen[counter]]].
          Chosen:=true;
      exit;
    end
    else
    begin
      checking:=false;

//those states that are unique for a single subgroup are eliminated. If a
//subgroup contain only these states, it is eliminated, change is counted and
//states of such subgroups are added to states of group[GroupNumber].
      for counter:=0 to high (group[GroupNumber].IncludedGroups) do
          if group[group[GroupNumber].IncludedGroups[counter]].Chosen=true then
      begin
        NewStates:=copy(NewStates, 0, 0);
        for counter2:=0 to high(group[group[GroupNumber].
            IncludedGroups[counter]].PositionStates) do
            for counter3:=0 to high (intersection) do
            if intersection[counter3]=group[group[GroupNumber].
            IncludedGroups[counter]].PositionStates[counter2] then
        begin
          setlength (NewStates, length(NewStates)+1);
          NewStates[high(NewStates)]:=intersection[counter3];
        end;
        if length (NewStates) = 0 then
        begin
          PositionChanges[Position]:=PositionChanges[Position]+1;
          setlength (TempDechosen, length (TempDechosen)+1);
          TempDechosen[high(TempDechosen)]:=counter;
          group[group[GroupNumber].IncludedGroups[counter]].Chosen:=false;
          for counter2:=0 to high (group[group[GroupNumber].
              IncludedGroups[counter]].PositionStates) do
          begin
            setlength (Group[GroupNumber].PositionStates,
                length (Group[GroupNumber].PositionStates)+1);
            Group[GroupNumber].PositionStates[high(Group[GroupNumber].
                PositionStates)]:=
                group[group[GroupNumber].
                IncludedGroups[counter]].PositionStates[counter2];
          end;
        end
        else if length (NewStates) <> length (Group[Group[GroupNumber].
            IncludedGroups[counter]].PositionStates) then
        begin
          checking:=true;
          setlength (Group[Group[GroupNumber].IncludedGroups[counter]].
              PositionStates, length (NewStates));
          for counter2:=0 to high(NewStates) do
              Group[Group[GroupNumber].IncludedGroups[counter]].
              PositionStates[counter2]:=NewStates[counter2];
        end;
      end;

//subgroups containing all states of any other subgroup are eliminated from
//further computing of changes.
      for counter:=0 to high (group[GroupNumber].IncludedGroups) do
          if group[group[GroupNumber].IncludedGroups[counter]].Chosen=true then
          for counter2:=0 to high (group[GroupNumber].IncludedGroups) do
          if counter <> counter2 then
          if group[group[GroupNumber].IncludedGroups[counter2]].Chosen=true then
      begin
        PassedChecking:=true;
        for counter3:=0 to high (group[group[GroupNumber].
            IncludedGroups[counter]].PositionStates) do
        begin
          passed:=false;
          for counter4:=0 to high (group[group[GroupNumber].
              IncludedGroups[counter2]].PositionStates) do
              if group[group[GroupNumber].IncludedGroups[counter]].
              PositionStates[counter3] =
              group[group[GroupNumber].IncludedGroups[counter2]].
              PositionStates[counter4] then passed:=true;
          if passed=false then PassedChecking:=false;
        end;
        if PassedChecking=true then
        begin
          checking:=true;
          setlength (TempDechosen, length (TempDechosen)+1);
          TempDechosen[high(TempDechosen)]:=counter2;
          group[group[GroupNumber].IncludedGroups[counter2]].Chosen:=false;
          Break;
          Continue;
        end;
      end;
    end;
  until checking=false;

//if the shortcuts could not solve the problem, the "remnants" of states of
//subgroups are copied to States and SetOrder procedure is called.
  States:=copy(States,0,0);
  for counter:=0 to high (group[GroupNumber].IncludedGroups) do
      if group[group[GroupNumber].IncludedGroups[counter]].Chosen = true then
      for counter2:=0 to high (group[group[GroupNumber].IncludedGroups[counter]].
      PositionStates) do
  begin
    checking:=false;
    for counter3:=0 to high (States) do if States[counter3].Symbol =
        group[group[GroupNumber].IncludedGroups[counter]].
        PositionStates[counter2] then checking:=true;
    if checking=false then
    begin
      setlength (States, length(States) + 1);
      States[high(States)].Symbol:= group[group[GroupNumber].
          IncludedGroups[counter]].PositionStates[counter2];
    end;
  end;
  for counter:=0 to high (States) do States[counter].Order:=-1;
  Minchanges:=-1;
  SetOrder (0, GroupNumber, States);
  PositionChanges[Position]:=PositionChanges[Position]+MinChanges;
  for counter:=0 to high (TempDechosen) do
      group[group[GroupNumber].IncludedGroups[TempDechosen[counter]]].
      Chosen:=true;
end;

//Helps with finding which groups were used to count chnges so that
//they can be writen in log file
procedure WriteGroup (GroupNumber: integer; UsedGroupsNumber: integer);
var number, counter: integer;
begin
  if Group[GroupNumber].chosen=true then
  begin
    if length (Group[GroupNumber].Taxa)>0 then
        UsedGroups[UsedGroupsNumber]:=UsedGroups[UsedGroupsNumber] +
        taxon[Group[GroupNumber].Taxa[0]].Name
    else
    begin
      UsedGroups[UsedGroupsNumber]:=UsedGroups[UsedGroupsNumber] +'(';
      number:=0;
      for counter:=0 to high (Group[GroupNumber].IncludedGroups)
          do number:=number+1;
      for counter:=0 to number - 2 do
      begin
        WriteGroup(Group[GroupNumber].IncludedGroups[counter],UsedGroupsNumber);
        UsedGroups[UsedGroupsNumber]:=UsedGroups[UsedGroupsNumber] +',';
      end;
      WriteGroup (Group[GroupNumber].IncludedGroups[number-1],UsedGroupsNumber);
      UsedGroups[UsedGroupsNumber]:=UsedGroups[UsedGroupsNumber] +')';
    end;
  end;
end;

//information on used groups and numbers of changes of all alignment positions
//is added to logfile
procedure ChangesLogFile (Changes: array of boolean);
var
  LogFile: textFile;
  counter, counter2, counter3: integer;
  checking: boolean;
begin
  Assign (LogFile, LogFileName);
  Append (LogFile);
  if ChangesLoaded = true then Writeln (LogFile, 'Changes loaded from a file')
      else Writeln (LogFile, 'Groups used for changes counting:');
  for counter:=0 to high (Group) do
      if group[counter].Chosen=true then
  begin
    checking:=false;
    for counter2:=0 to counter - 1 do
        if group[counter2].Chosen=true then
        for counter3:=0 to high (group[counter2].IncludedGroups) do
        if Group[counter2].IncludedGroups[counter3] = counter then
        checking:=true;
    if checking=false then
    begin
      SetLength (UsedGroups, length (UsedGroups)+1);
      WriteGroup (counter, high (UsedGroups));
    end;
  end;
  for counter:=0 to high (UsedGroups) do
      Writeln (LogFile, UsedGroups[counter]);
  Writeln (LogFile);
  For counter:=0 to high (Changes) do
  begin
    Write (LogFile, 'Positions with ' + IntToStr(counter) + ' changes: ');
    for counter2:=0 to high (PositionChanges) do
        if PositionChanges[counter2] = counter then
        Write (LogFile, IntToStr (counter2 + 1) + ' ');
    Writeln (LogFile);
  end;
  Writeln (Logfile);
  Writeln (Logfile, 'Number of changes for all alignment positions: ');
  for counter:=0 to high (PositionChanges) do
  begin
    if (counter mod 10 = 0) and (counter > 0) then writeln (LogFile);
    if (counter mod 100 = 0) and (counter > 0) then writeln (LogFile);
    write (LogFile,
        IntToStr(PositionChanges[counter]) + ' ');
  end;
  CloseFile(LogFile);
end;

//Creates a new alignment file with positions with more changes than treshold
//deleted.
procedure NewAlignment (ChangeNumber: integer; labeled: string; format: TFormat);
var NewAlignment: TextFile;
  LastData, counter, counter2, TaxonCounter, DataCounter: integer;
  NewAlignmentName: string;
begin
  NewAlignmentName:=AlignmentFileName;
  for counter:= length(NewAlignmentName)-1 downto 0 do
      if NewAlignmentName[counter] = '.' then
  begin
    NewAlignmentName:=copy(NewAlignmentName,0,counter-1) + labeled;
    break;
  end
  else if counter=0 then NewAlignmentName:=NewAlignmentName + labeled;
  if format = NEXUS then
  begin
    NewAlignmentName:=NewAlignmentName + '.nex';
    Assign (NewAlignment, NewAlignmentName);
    Rewrite (NewAlignment);
    writeln (NewAlignment, '#NEXUS');
    Writeln (NewAlignment);
    Writeln (NewAlignment, 'Begin data;');
    counter:=0;
    for DataCounter:=0 to high (Taxon[0].Data) do
        if PositionChanges[DataCounter]<ChangeNumber+1 then
        counter:=counter+1;
    Writeln (NewAlignment, '  Dimensions ntax=' + IntToStr (length(Taxon)) +
        ' nchar=' + IntToStr (counter) + ';');
    if DataType = DNA then writeln (NewAlignment, '  Format datatype=DNA gap = -;')
        else writeln (NewAlignment, '  Format datatype=Protein gap = -;');
    Writeln (NewAlignment, 'matrix');
    for TaxonCounter:=0 to high (Taxon) do
    begin
      counter:=0;
      Write (NewAlignment, Taxon[TaxonCounter].Name);
      for DataCounter:=0 to high (Taxon[0].Data) do
          if PositionChanges[DataCounter]<ChangeNumber+1 then
      begin
        if counter mod 50 = 0 then writeln (NewAlignment);
        counter:=counter+1;
        Write (NewAlignment, Taxon[TaxonCounter].Data[DataCounter])
      end;
      writeln (NewAlignment);
    end;
    writeln (NewAlignment, ';');
    writeln (NewAlignment, 'end;');
    CloseFile (NewAlignment);
  end
  else if format = FASTA then
  begin
    NewAlignmentName:=NewAlignmentName + '.fas';
    Assign (NewAlignment, NewAlignmentName);
    Rewrite (NewAlignment);
    for TaxonCounter:=0 to high (Taxon) do
    begin
      Write (NewAlignment, '>' + Taxon[TaxonCounter].Name);
      counter:=0;
      for DataCounter:=0 to high (Taxon[0].Data) do
          if PositionChanges[DataCounter]<ChangeNumber+1 then
      begin
        if counter mod 50 = 0 then writeln (NewAlignment);
        counter:=counter+1;
        Write (NewAlignment, Taxon[TaxonCounter].Data[DataCounter])
      end;
      Writeln (NewAlignment);
    end;
    CloseFile (NewAlignment);
  end
  else
  begin
    NewAlignmentName:=NewAlignmentName + '.phy';
    Assign (NewAlignment, NewAlignmentName);
    Rewrite (NewAlignment);
    counter:=0;
    for DataCounter:=0 to high (Taxon[0].Data) do
        if PositionChanges[DataCounter]<ChangeNumber+1 then
        counter:=counter+1;
    Write (NewAlignment, ' ' + inttostr (length (Taxon)) + ' ' + inttostr (counter));
    for TaxonCounter:=0 to high (Taxon) do
    begin
      Writeln (NewAlignment);
      Write (NewAlignment, Taxon[TaxonCounter].Name);
      for counter:=0 to 11-length (Taxon[TaxonCounter].Name) do write (NewAlignment, ' ' );
      DataCounter:=0;
      counter:=0;
      repeat
        if PositionChanges[DataCounter]<ChangeNumber+1 then
        begin
          if counter mod 10 = 0 then write (NewAlignment, ' ');
          Write (NewAlignment, Taxon[TaxonCounter].Data[DataCounter]);
          counter:=counter+1;
        end;
        DataCounter:=DataCounter+1;
      until (counter = 50) or (DataCounter=length (Taxon[0].Data));
      LastData:=DataCounter;
    end;
    if DataCounter < length (taxon[0].Data) then
    repeat
      writeln (NewAlignment);
      for TaxonCounter:=0 to high (Taxon) do
      begin
        Writeln (NewAlignment);
        DataCounter:=LastData;
        counter:=0;
        Write (NewAlignment, '            ');
        repeat
          if PositionChanges[DataCounter]<ChangeNumber+1 then
          begin
            if counter mod 10 = 0 then write (NewAlignment, ' ');
            Write (NewAlignment, Taxon[TaxonCounter].Data[DataCounter]);
            counter:=counter+1;
          end;
          DataCounter:=DataCounter+1;
        until (counter = 50) or (DataCounter=length (Taxon[0].Data));
      end;
      LastData:=DataCounter;
    until DataCounter = length (Taxon[0].Data);
    CloseFile (NewAlignment);
  end;
end;

//Calls procedures that count changes and creates new data files.
procedure TForm1.Button3Click(Sender: TObject);
var
  DataCounter, GroupCounter, counter, ChosenGroupsNumber: integer;
  Changes: array of boolean;
  labeled: string;
  format: TFormat;
begin
  if ChangesLoaded = false then
  begin
    RadioGroup2.enabled:=false;
    Button3.enabled:=false;
    label1.enabled:=false;
    label2.visible:=true;
    label2.caption:='Progress: counting changes';
    Application.Processmessages;
    Setlength (PositionChanges, length (Taxon[0].Data));
    for DataCounter:=0 to high(Taxon[0].Data) do
    begin
      PositionChanges[DataCounter]:=0;
      for GroupCounter:=high(group) downto 0 do
          if Group[GroupCounter].Chosen=true then
      begin
        group[GroupCounter].PositionStates:=copy(group[GroupCounter].
            PositionStates,0,0);
        if length (Group[GroupCounter].Taxa)>0 then
            case Datatype of
              DNA: SetStateDNA (GroupCounter, DataCounter);
              AA: SetStateAA (GroupCounter, DataCounter);
            end
            else
        begin
          ChosenGroupsNumber:=0;
          for counter:=0 to high(Group[GroupCounter].IncludedGroups) do
              if Group[Group[GroupCounter].IncludedGroups[counter]].Chosen = true
              then ChosengroupsNumber:=ChosenGroupsNumber+1;
          case ChosenGroupsNumber of
            0: Group[GroupCounter].Chosen:=false;
            1: CopyStates (GroupCounter);
            2: CheckChanges (GroupCounter, DataCounter);
          else CheckChangesPoly (Groupcounter, DataCounter);
          end;
        end;
      end;
    end;
  end;
  Setlength (Changes, 1);
  Changes[0]:=true;
  for counter:=0 to high (PositionChanges) do
  begin
    if PositionChanges[counter]>high(Changes) then Setlength (Changes,
        PositionChanges[counter]+1);
    Changes[PositionChanges[counter]]:=true;
  end;
  case RadioGroup2.ItemIndex of
    0: format:=NEXUS;
    1: format:=FASTA;
    2: format:=Phylip;
  end;
  if InputFormat <> format then
      NewAlignment (high (Changes), '_AllData', format);
  for counter:=high(Changes)-1 downto 0 do if Changes[counter]=true then
  begin
    label2.caption:='Progress: saving data - ' + IntToStr (counter) + ' changes';
    Application.Processmessages;
    labeled:='_S' + IntToStr (counter);
    NewAlignment (counter, labeled, format);
  end;
  if length (changes) = 1 then
  begin
    label2.caption:='Progress: saving data - 0 changes';
    Application.Processmessages;
    NewAlignment (0, '_S0', format);
  end;
  label2.caption:='Progress: finishing log file';
  Application.Processmessages;
  ChangesLogFile (Changes);
  Application.terminate;
end;

//direct loading of changes number from a file
procedure TForm1.Button4Click(Sender: TObject);
var ChangesFile: textFile;
  Symbol: char;
  counter: integer;
  Word: string;
begin
  setlength (PositionChanges, length (Taxon[0].Data));
  counter:=0;
  if OpenDialog3.Execute then
  begin
    AssignFile (ChangesFile, OpenDialog3.FileName);
    Reset (ChangesFile);
    repeat
      if not eof (ChangesFile) then if eoln (ChangesFile) then
      begin
        if Word <> '' then
        begin
          PositionChanges[counter]:=strtoint (Word);
          Word:='';
          counter:=counter+1;
        end;
        readln (ChangesFile);
        continue;
      end;
      read (ChangesFile, Symbol);
      if eof (ChangesFile) then if Symbol <> ' ' then
      begin
        PositionChanges[counter]:=strtoint (Word + Symbol);
        counter:=counter+1;
      end;
      if symbol = ' ' then
      begin
        if Word <> '' then
        begin
          PositionChanges[counter]:=strtoint(Word);
          counter:=counter + 1;
          Word:='';
        end;
      end
      else Word :=Word + Symbol;
      if counter > length (Taxon[0].Data) then
      begin
        PositionChanges:=nil;
        MessageDlg
            ('More data in changes file then alignment positions.',
            mtError, [mbOK], 0);
        CloseFile (ChangesFile);
        exit;
      end;
    until eof (ChangesFile);
    CloseFile (ChangesFile);
    if counter < length (Taxon[0].Data) then
    begin
      PositionChanges:=nil;
      MessageDlg
          ('Not enough data in changes file (less than alignment positions).',
          mtError, [mbOK], 0);
      exit;
    end;
    Button2.Enabled:=false;
    Button4.Enabled:=false;
    Button3.Enabled:=true;
    RadioGroup2.Enabled:=true;
    Memo1.Lines.Clear;
    Memo1.Lines.Append('  Hints:');
    Memo1.Lines.Append(' ');
    Memo1.Lines.Append('Finaly, specify the format you');
    Memo1.Lines.Append('want your new datasets saved');
    Memo1.Lines.Append('in and click on the New alignments');
    Memo1.Lines.Append('button.');
    ChangesLoaded:=true;
  end;
end;

//"reset" button
procedure TForm1.Button5Click(Sender: TObject);
begin
  if MessageDlg ('Warning: this will reset the program and allow you to load new alignment.',
      mtWarning, [mbYes,mbNo], 0) = mrYes then
  begin
    Radiogroup1.Enabled:=true;
    Button1.Enabled:=true;
    Button2.Enabled:=false;
    Button3.Enabled:=false;
    Button4.Enabled:=false;
    Label1.Enabled:=false;
    RadioGroup2.Enabled:=false;
    Image1.Visible:=false;
    Image1.Canvas.Pen.Color:=clWhite;
    Image1.Canvas.Brush.Color:=clWhite;
    Image1.Canvas.Rectangle (0,0,Image1.Width, Image1.Height);
    Taxon:=nil;
    Group:=nil;
    PositionChanges:=nil;
    SymbolsUsed:=nil;
    CountChangesSymbols:=nil;
    UsedGroups:=nil;
    CanResizeTree:= false;
    AssignedTaxa:= nil;
    ChangesLoaded:= false;
    maxGroupLevel:=0;
    Memo1.Lines.Clear;
    Memo1.Lines.Append ('  Hints:                           ');
    Memo1.Lines.Append ('');
    Memo1.Lines.Append ('First, specify the data type:');
    Memo1.Lines.Append ('DNA or proteins (DNA is default).');
    Memo1.Lines.Append ('');
    Memo1.Lines.Append ('Then click on the Load alignment');
    Memo1.Lines.Append ('button to load your data in Phylip,');
    Memo1.Lines.Append ('FASTA or NEXUS format.');
    Memo1.Lines.Append ('You may check sample files if a');
    Memo1.Lines.Append ('problem occurs.');
    Memo1.Lines.Append ('');
    Memo1.Lines.Append ('[For DNA and protein alignments,');
    Memo1.Lines.Append ('standard ambiguity codes are');
    Memo1.Lines.Append ('supported, with"-" for gaps and');
    Memo1.Lines.Append ('"?" for unknown. It means "N"');
    Memo1.Lines.Append ('(DNA) of X (Protein) stands for');
    Memo1.Lines.Append ('unknown nucleotide / AA, but');
    Memo1.Lines.Append ('"?" stands for unknown');
    Memo1.Lines.Append ('nucleotide / AA or gap.]');
  end;
end;

end.
